/*******************************************************************************
 * Copyright (c) 2010 Thales Corporate Services SAS                             *
 * Author : Gregory Boissinot                                                   *
 *                                                                              *
 * Permission is hereby granted, free of charge, to any person obtaining a copy *
 * of this software and associated documentation files (the "Software"), to deal*
 * in the Software without restriction, including without limitation the rights *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell    *
 * copies of the Software, and to permit persons to whom the Software is        *
 * furnished to do so, subject to the following conditions:                     *
 *                                                                              *
 * The above copyright notice and this permission notice shall be included in   *
 * all copies or substantial portions of the Software.                          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE  *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER       *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,*
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN    *
 * THE SOFTWARE.                                                                *
 *******************************************************************************/

package com.thalesgroup.dtkit.ws.rs.providers;

import org.codehaus.jackson.map.ObjectMapper;

import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;

import com.thalesgroup.dtkit.ws.rs.vo.InputMetricsResult;
import com.thalesgroup.dtkit.ws.rs.vo.InputMetricResult;

@Provider
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
public class InputMetricJSONProvider implements MessageBodyWriter, MessageBodyReader<InputMetricsResult> {

    @Override
    public long getSize(Object obj, Class type, Type genericType,
                        Annotation[] annotations, MediaType mediaType) {
        //Returns -1 (can't be determined in advance)
        return -1;
    }

    @Override
    public boolean isWriteable(Class type, Type genericType,
                               Annotation annotations[], MediaType mediaType) {
        return true;
    }

    @Override
    public void writeTo(Object target, Class type, Type genericType,
                        Annotation[] annotations, MediaType mediaType,
                        MultivaluedMap httpHeaders, OutputStream outputStream)
            throws IOException {

        //Use Jackson to build the json output and write it in the outputStream object
        ObjectMapper objectMapper = new ObjectMapper();
        Object entity = target;

        if (type == InputMetricResult.class) {
            objectMapper = objectMapper.enableDefaultTyping();
            entity = ((InputMetricResult) target).getInputMetric();
        }
        if (type == InputMetricsResult.class) {
            objectMapper = objectMapper.enableDefaultTyping();
            entity = ((InputMetricsResult) target).getMetrics();
        }

        objectMapper.writeValue(outputStream, entity);
    }

    @Override
    public boolean isReadable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return (type == InputMetricResult.class || type == InputMetricsResult.class);
    }

    @Override
    public InputMetricsResult readFrom(Class<InputMetricsResult> type,
                                       Type genericType,
                                       Annotation[] annotations,
                                       MediaType mediaType,
                                       MultivaluedMap<String, String> httpHeaders,
                                       InputStream entityStream) throws IOException, WebApplicationException {
        return new ObjectMapper().enableDefaultTyping().readValue(entityStream, type);
    }
}

